/*
* This file is part of fwmon
* Copyright (c) 2001 Gianni Tedesco
*
*    fwmon is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    fwmon is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with fwmon; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "libpcap.h"

int cap_fd=-1;
static const char rcsid[]=
	"@(#) $Header: /data/cvsroot/fwmon/libpcap.c,v 1.5 2001/12/29 17:29:41 gianni Exp $";

int lpcap_init(void)
{
	struct pcap_file_header phdr;
	int fe,flags=O_WRONLY | O_CREAT;

	if ( !flag(FLAG_LIBPCAP) ) return 1;

	/* RACE! Should open, then fstat, then ftruncate() */
	fe = lpcap_file_exists(globals.pcapfile);

	if ( !fe ) {
		flags |= O_TRUNC;
	}else{
		flags |= O_APPEND;
	}
	
	cap_fd = open(globals.pcapfile, flags, 00600);
	if ( cap_fd < 0 ) {
		perror("opening tcpdump file" CRLF);
		return 0;
	}


	/* Setup the header if the file didnt
	 * already exist. */
	if ( !fe ) {
		phdr.magic = 0xa1b2c3d4; /* Stolen from snort (??) */
		phdr.version_major = PCAP_VERSION_MAJOR;
		phdr.version_minor = PCAP_VERSION_MINOR;
		phdr.thiszone = 0;
		phdr.sigfigs = 0;
		phdr.snaplen = BUFFER_SIZE;
		phdr.linktype = DLT_RAW;

		if ( write(cap_fd, &phdr, sizeof(phdr)) < 0 ) {
			perror("writing pcap header");
			return 0;
		}
		
		fsync(cap_fd);
	}

	return 1;
}

/* is this file a valid pcap file? If not, clobber it. 
 * FIXME: Is clobbering the file bad behaivour? */
int lpcap_file_exists(const char *fn)
{
	int tmp_fd;
	struct stat sfile;
	struct pcap_file_header phdr;
	
	if ( stat(fn, &sfile) != 0 ) return 0;

	tmp_fd = open(fn, O_RDONLY);
	if ( tmp_fd < 0 ) return 0;

	if ( read(tmp_fd, &phdr, sizeof(phdr)) < sizeof(phdr) ) {
		close(tmp_fd);
		return 0;
	}

	close(tmp_fd);

	if ( ! ((phdr.magic == 0xa1b2c3d4) && 
		(phdr.linktype == DLT_RAW)) ) return 0;
	
	return 1;
}

void lpcap_close(void)
{
	if ( !flag(FLAG_LIBPCAP) ) return;
	if ( cap_fd >= 0 ) {
		fsync(cap_fd);
		close(cap_fd);
		cap_fd = -1;
	}
}

void lpcap_dump ( p_packet p )
{
	struct pcap_pkthdr phdr;

	phdr.caplen	= p->data.netlink.len-sizeof(p->data.netlink);
	phdr.len 	= ntohs(p->ip->tot_len);
	gettimeofday( (struct timeval *)&phdr.ts, NULL );

	write(cap_fd, &phdr, sizeof(phdr) );
	write(cap_fd, p->ip, phdr.caplen );
	fsync(cap_fd);
}
